@@ -2,20 +2,22 @@
  * GNU LibreJS - A browser add-on to block nonfree nontrivial JavaScript.
  * *
  * Copyright (C) 2011, 2012, 2013, 2014 Loic J. Duros
+ * Copyright (C) 2014, 2015 Nik Nyby
  *
- * This program is free software: you can redistribute it and/or modify
+ * This file is part of GNU LibreJS.
+ *
+ * GNU LibreJS is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * GNU LibreJS is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see  <http://www.gnu.org/licenses/>.
- *
+ * along with GNU LibreJS.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 // node.js url module. Makes it easier to resolve
@@ -148,24 +150,102 @@ WebLabelFinder.prototype.fetchLicensePage = function() {
     }
 };
 
+/**
+ * @method isLicenseFree
+ * Returns true if the given web labels row refers to a script that
+ * can be executed by LibreJS.
+ *
+ * This method has some side effects :-/
+ *
+ * @param {Object} lic - A license node from a JS web labels page. It's
+ *                       expected to contain one or more licenses.
+ * @return {Boolean}
+ */
+WebLabelFinder.prototype.isLicenseFree = function(
+    lic, jslicenseURL, callback
+) {
+    // For each license that this license row contains.
+    var isFree = false;
+    // licenseStatuses is later used to determine isFree.
+    var licenseStatuses = [];
+
+    for (var i = 0; i < lic.licenses.length; i++) {
+        var license;
+        var found = false;
+
+        // For each license from the internal license definitions
+        for (license in licenses) {
+            if (found === true) {
+                break;
+            }
+            var licDef = licenses[license];
+            var licArray = [];
+            if (!licDef.canonicalUrl) {
+                continue;
+            }
+            if (typeof licDef.canonicalUrl === 'string') {
+                licArray = [licDef.canonicalUrl];
+            } else {
+                licArray = licDef.canonicalUrl;
+            }
+
+            // For each of the canonical URLs recognized by this license
+            // definition
+            for (var j = 0; j < licArray.length; j++) {
+                if (urlHandler.removeFragment(licArray[j]) ===
+                    urlHandler.removeFragment(lic.licenses[i].licenseUrl)
+                   ) {
+                    if (!require("sdk/url").isValidURI(lic.fileUrl)) {
+                        console.debug(lic.fileUrl, " is not a valid URL");
+                        callback();
+                    }
+
+                    // This license was recognized, and it was free. Add it
+                    // to the array of license status, which we'll look at
+                    // when we're done with this web label row.
+                    licenseStatuses.push(true);
+
+                    console.debug("about TO ADD TO XHR: ", lic.fileUrl);
+                    this.listCheck[lic.fileUrl] = 0;
+                    addToCache(lic, 0, jslicenseURL, callback);
+
+                    // Break out of the nearest two loops cause we found
+                    // a matching license
+                    found = true;
+                    break;
+                }
+            }
+        }
+    }
+
+    // Tally up the licenses we were able to match.
+    if (licenseStatuses.length > 0 &&
+        // If the number of licenses we matched is at least one, and
+        // it's the same number as the number of licenses in this Web
+        // Label column, only then can we recognize this script as free.
+        // licenseStatus.length should never be larger than
+        // lic.licenses.length.
+        licenseStatuses.length >= lic.licenses.length
+       ) {
+        isFree = true;
+    }
+
+    return isFree;
+};
+
 WebLabelFinder.prototype.matchListWithDefs = function(jslicenseURL) {
-    var i = 0,
-        len = this.licenseList.length,
-        lic,
-        licDef,
-        urlLength,
-        iUrl,
-        licArray = null,
+    var that = this;
+    var licDef,
         license, script;
     var cacheCalls = 0;
-    list_check = {};
+    this.listCheck = {};
 
     // nested loop.
     cacheCalls = 0;
     var callback = function (url) {
         cacheCalls++;
-        list_check[url] = 1;
-        if (cacheCalls === Object.keys(list_check).length) {
+        that.listCheck[url] = 1;
+        if (cacheCalls === Object.keys(that.listCheck).length) {
             console.debug("triggering callback duh");
             // return array to requester object
             callback = false;
@@ -178,41 +258,17 @@ WebLabelFinder.prototype.matchListWithDefs = function(jslicenseURL) {
         // return array to requester object
         if (callback !== false) {
             that.callback(that.licenseList);
-            console.debug(list_check);
+            console.debug(that.listCheck);
         }
     }, 15000);
 
-    for (; i < len; i++) {
-        lic = this.licenseList[i];
-        var that = this;
-        for (license in licenses) {
-            licDef = licenses[license];
-            if (licDef.canonicalUrl !== undefined) {
-                if (typeof licDef.canonicalUrl === 'string') {
-                    // this is a string. make it an array.
-                    licArray = [licDef.canonicalUrl];
-                }  else {
-                    licArray = licDef.canonicalUrl;
-                }
 
-                urlLength = licArray.length;
 
-                for (iUrl = 0; iUrl < urlLength; iUrl++) {
-                    if (urlHandler.removeFragment(licArray[iUrl]) ===
-                        urlHandler.removeFragment(lic.licenseUrl)) {
-                        if (!require("sdk/url").isValidURI(lic.fileUrl)) {
-                            console.debug(lic.fileUrl, " is not a valid URL");
-                            callback();
-                        }
-                        lic.free = true;
-                        var notif = require("ui/notification")
-                            .createNotification(lic.fileUrl).notification;
-                        console.debug("about TO ADD TO XHR: ", lic.fileUrl);
-                        list_check[lic.fileUrl] = 0;
-                        addToCache(lic, 0, jslicenseURL, callback);
-                    }
-                }
-            }
+    for (var i = 0; i < this.licenseList.length; i++) {
+        // this.licenseList[i] is the web labels license column
+        var lic = this.licenseList[i];
+        if (this.isLicenseFree(lic, jslicenseURL, callback)) {
+            lic.free = true;
         }
     }
 };
